Package: ess
Title: Efficient Stepwise Selection in Decomposable Models
Version: 1.1.2.1
Authors@R: 
    person(given = "Mads",
           family = "Lindskou",
           role = c("aut", "cre"),
           email = "mads@math.aau.dk")
Description: An implementation of the ESS algorithm following Amol Deshpande, Minos Garofalakis,
	     Michael I Jordan (2013) <doi:10.48550/arXiv.1301.2267>. The ESS algorithm
	     is used for model selection in decomposable graphical models.
URL: https://github.com/mlindsk/ess
Depends: R (>= 3.5.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
Imports: Rcpp, igraph
LinkingTo: Rcpp
RoxygenNote: 7.1.1
Suggests: tinytest
BugReports: https://github.com/mlindsk/ess/issues
NeedsCompilation: yes
Packaged: 2026-01-10 16:04:12 UTC; hornik
Author: Mads Lindskou [aut, cre]
Maintainer: Mads Lindskou <mads@math.aau.dk>
Repository: CRAN
Date/Publication: 2026-01-10 16:08:14 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 14:54:01 UTC; unix
Archs: ess.so.dSYM
