Package: ess
Version: 5.1.13-2
Section: math
Priority: optional
Architecture: all
Depends: emacsen
Conflicts: dhelp (<= 0.3.12)
Installed-Size: 1137
Maintainer: Camm Maguire <camm@enhanced.com>
Description: Emacs statistics mode, supporting R,S and others
 ESS is an Emacs package which provides a standard interface between
 statistical programs and statistical processes.  It is intended to
 provide assistance for interactive statistical programming and data
 analysis, and is based on and extends the capabilities of S-mode.
