Test



Literate Data Analysis is a concept that takes Literate Programming
and implements it for data analysis.  We envision it as a means to
record for future use, the trials and tribulations of data analysis
which are helpful for future statisticians, and to provide examples of
expert analyses (including mistakes and the roads less and not taken).

This directory contains work on a tech report for ESS and literate
data analysis, as well as examples of literary styles and
applications.

Files:
- ex1.nw : trivial example of the use of multiple code modes
- ex2.nw :
- lda-ess.tex (.pdf, .ps) : Tech Report
