How do we organize the  ess/doc/ directory ?
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Source: *.texi  [texinfo]

Targets (see also ./Makefile):

	  ess.dvi	[DVI -> PostScript]
	  readme.dvi

	  ess.info*	[Info]
	  readme.info*

	  ./html/*html

----------------------

Structure:  TWO main files

	    (note that a third one, announc.texi,
	     is conceptually a subset of readme.texi and
	     is only used to automatically produce ../ANNOUNCE)

 ess.texi
  |
  |-- ../VERSION
  |-- ess-defs.texi
  |
  |-- newfeat.texi
  |
  |-- credits.texi
  |      \-- authors.texi
  |
  |-- getting.texi
  |      \-- inst_svn.texi
  |-- inst_tar.texi
  |
  |-- requires.texi
  |
  |-- help-s.texi
  |-- help-sas.texi
  |-- help-bugs.texi
  |
  |-- bugs.texi
  |
  |-- bugrept.texi
  |
  \-- mailing.texi




 readme.texi
  |
  |-- license.texi
  |
  |-- newfeat.texi
  |
  |-- currfeat.texi
  |
  |-- stabilty.texi
  |
  |-- requires.texi
  |
  |-- getting.texi
  |      \-- inst_svn.texi
  |-- inst_tar.texi
  |
  |-- bugrept.texi
  |
  |-- mailing.texi
  |
  \-- authors.texi

